@@ -231,11 +231,7 @@ static char *(features[]) =
 #else
 	"-dnd",
 #endif
-#ifdef EBCDIC
-	"+ebcdic",
-#else
 	"-ebcdic",
-#endif
 #ifdef FEAT_EMACS_TAGS
 	"+emacs_tags",
 #else
@@ -379,7 +375,11 @@ static char *(features[]) =
 	"-mouse_dec",
 # endif
 # ifdef FEAT_MOUSE_GPM
+#  ifdef DYNAMIC_GPM
+	"+mouse_gpm/dyn",
+#  else
 	"+mouse_gpm",
+#  endif
 # else
 	"-mouse_gpm",
 # endif
@@ -552,7 +552,11 @@ static char *(features[]) =
 	"-smartindent",
 #endif
 #ifdef FEAT_SODIUM
+# ifdef DYNAMIC_SODIUM
+	"+sodium/dyn",
+# else
 	"+sodium",
+# endif
 #else
 	"-sodium",
 #endif
@@ -750,6 +754,994 @@ static char *(features[]) =
 
 static int included_patches[] =
 {   /* Add new patch number below this line */
+/**/
+    4489,
+/**/
+    4488,
+/**/
+    4487,
+/**/
+    4486,
+/**/
+    4485,
+/**/
+    4484,
+/**/
+    4483,
+/**/
+    4482,
+/**/
+    4481,
+/**/
+    4480,
+/**/
+    4479,
+/**/
+    4478,
+/**/
+    4477,
+/**/
+    4476,
+/**/
+    4475,
+/**/
+    4474,
+/**/
+    4473,
+/**/
+    4472,
+/**/
+    4471,
+/**/
+    4470,
+/**/
+    4469,
+/**/
+    4468,
+/**/
+    4467,
+/**/
+    4466,
+/**/
+    4465,
+/**/
+    4464,
+/**/
+    4463,
+/**/
+    4462,
+/**/
+    4461,
+/**/
+    4460,
+/**/
+    4459,
+/**/
+    4458,
+/**/
+    4457,
+/**/
+    4456,
+/**/
+    4455,
+/**/
+    4454,
+/**/
+    4453,
+/**/
+    4452,
+/**/
+    4451,
+/**/
+    4450,
+/**/
+    4449,
+/**/
+    4448,
+/**/
+    4447,
+/**/
+    4446,
+/**/
+    4445,
+/**/
+    4444,
+/**/
+    4443,
+/**/
+    4442,
+/**/
+    4441,
+/**/
+    4440,
+/**/
+    4439,
+/**/
+    4438,
+/**/
+    4437,
+/**/
+    4436,
+/**/
+    4435,
+/**/
+    4434,
+/**/
+    4433,
+/**/
+    4432,
+/**/
+    4431,
+/**/
+    4430,
+/**/
+    4429,
+/**/
+    4428,
+/**/
+    4427,
+/**/
+    4426,
+/**/
+    4425,
+/**/
+    4424,
+/**/
+    4423,
+/**/
+    4422,
+/**/
+    4421,
+/**/
+    4420,
+/**/
+    4419,
+/**/
+    4418,
+/**/
+    4417,
+/**/
+    4416,
+/**/
+    4415,
+/**/
+    4414,
+/**/
+    4413,
+/**/
+    4412,
+/**/
+    4411,
+/**/
+    4410,
+/**/
+    4409,
+/**/
+    4408,
+/**/
+    4407,
+/**/
+    4406,
+/**/
+    4405,
+/**/
+    4404,
+/**/
+    4403,
+/**/
+    4402,
+/**/
+    4401,
+/**/
+    4400,
+/**/
+    4399,
+/**/
+    4398,
+/**/
+    4397,
+/**/
+    4396,
+/**/
+    4395,
+/**/
+    4394,
+/**/
+    4393,
+/**/
+    4392,
+/**/
+    4391,
+/**/
+    4390,
+/**/
+    4389,
+/**/
+    4388,
+/**/
+    4387,
+/**/
+    4386,
+/**/
+    4385,
+/**/
+    4384,
+/**/
+    4383,
+/**/
+    4382,
+/**/
+    4381,
+/**/
+    4380,
+/**/
+    4379,
+/**/
+    4378,
+/**/
+    4377,
+/**/
+    4376,
+/**/
+    4375,
+/**/
+    4374,
+/**/
+    4373,
+/**/
+    4372,
+/**/
+    4371,
+/**/
+    4370,
+/**/
+    4369,
+/**/
+    4368,
+/**/
+    4367,
+/**/
+    4366,
+/**/
+    4365,
+/**/
+    4364,
+/**/
+    4363,
+/**/
+    4362,
+/**/
+    4361,
+/**/
+    4360,
+/**/
+    4359,
+/**/
+    4358,
+/**/
+    4357,
+/**/
+    4356,
+/**/
+    4355,
+/**/
+    4354,
+/**/
+    4353,
+/**/
+    4352,
+/**/
+    4351,
+/**/
+    4350,
+/**/
+    4349,
+/**/
+    4348,
+/**/
+    4347,
+/**/
+    4346,
+/**/
+    4345,
+/**/
+    4344,
+/**/
+    4343,
+/**/
+    4342,
+/**/
+    4341,
+/**/
+    4340,
+/**/
+    4339,
+/**/
+    4338,
+/**/
+    4337,
+/**/
+    4336,
+/**/
+    4335,
+/**/
+    4334,
+/**/
+    4333,
+/**/
+    4332,
+/**/
+    4331,
+/**/
+    4330,
+/**/
+    4329,
+/**/
+    4328,
+/**/
+    4327,
+/**/
+    4326,
+/**/
+    4325,
+/**/
+    4324,
+/**/
+    4323,
+/**/
+    4322,
+/**/
+    4321,
+/**/
+    4320,
+/**/
+    4319,
+/**/
+    4318,
+/**/
+    4317,
+/**/
+    4316,
+/**/
+    4315,
+/**/
+    4314,
+/**/
+    4313,
+/**/
+    4312,
+/**/
+    4311,
+/**/
+    4310,
+/**/
+    4309,
+/**/
+    4308,
+/**/
+    4307,
+/**/
+    4306,
+/**/
+    4305,
+/**/
+    4304,
+/**/
+    4303,
+/**/
+    4302,
+/**/
+    4301,
+/**/
+    4300,
+/**/
+    4299,
+/**/
+    4298,
+/**/
+    4297,
+/**/
+    4296,
+/**/
+    4295,
+/**/
+    4294,
+/**/
+    4293,
+/**/
+    4292,
+/**/
+    4291,
+/**/
+    4290,
+/**/
+    4289,
+/**/
+    4288,
+/**/
+    4287,
+/**/
+    4286,
+/**/
+    4285,
+/**/
+    4284,
+/**/
+    4283,
+/**/
+    4282,
+/**/
+    4281,
+/**/
+    4280,
+/**/
+    4279,
+/**/
+    4278,
+/**/
+    4277,
+/**/
+    4276,
+/**/
+    4275,
+/**/
+    4274,
+/**/
+    4273,
+/**/
+    4272,
+/**/
+    4271,
+/**/
+    4270,
+/**/
+    4269,
+/**/
+    4268,
+/**/
+    4267,
+/**/
+    4266,
+/**/
+    4265,
+/**/
+    4264,
+/**/
+    4263,
+/**/
+    4262,
+/**/
+    4261,
+/**/
+    4260,
+/**/
+    4259,
+/**/
+    4258,
+/**/
+    4257,
+/**/
+    4256,
+/**/
+    4255,
+/**/
+    4254,
+/**/
+    4253,
+/**/
+    4252,
+/**/
+    4251,
+/**/
+    4250,
+/**/
+    4249,
+/**/
+    4248,
+/**/
+    4247,
+/**/
+    4246,
+/**/
+    4245,
+/**/
+    4244,
+/**/
+    4243,
+/**/
+    4242,
+/**/
+    4241,
+/**/
+    4240,
+/**/
+    4239,
+/**/
+    4238,
+/**/
+    4237,
+/**/
+    4236,
+/**/
+    4235,
+/**/
+    4234,
+/**/
+    4233,
+/**/
+    4232,
+/**/
+    4231,
+/**/
+    4230,
+/**/
+    4229,
+/**/
+    4228,
+/**/
+    4227,
+/**/
+    4226,
+/**/
+    4225,
+/**/
+    4224,
+/**/
+    4223,
+/**/
+    4222,
+/**/
+    4221,
+/**/
+    4220,
+/**/
+    4219,
+/**/
+    4218,
+/**/
+    4217,
+/**/
+    4216,
+/**/
+    4215,
+/**/
+    4214,
+/**/
+    4213,
+/**/
+    4212,
+/**/
+    4211,
+/**/
+    4210,
+/**/
+    4209,
+/**/
+    4208,
+/**/
+    4207,
+/**/
+    4206,
+/**/
+    4205,
+/**/
+    4204,
+/**/
+    4203,
+/**/
+    4202,
+/**/
+    4201,
+/**/
+    4200,
+/**/
+    4199,
+/**/
+    4198,
+/**/
+    4197,
+/**/
+    4196,
+/**/
+    4195,
+/**/
+    4194,
+/**/
+    4193,
+/**/
+    4192,
+/**/
+    4191,
+/**/
+    4190,
+/**/
+    4189,
+/**/
+    4188,
+/**/
+    4187,
+/**/
+    4186,
+/**/
+    4185,
+/**/
+    4184,
+/**/
+    4183,
+/**/
+    4182,
+/**/
+    4181,
+/**/
+    4180,
+/**/
+    4179,
+/**/
+    4178,
+/**/
+    4177,
+/**/
+    4176,
+/**/
+    4175,
+/**/
+    4174,
+/**/
+    4173,
+/**/
+    4172,
+/**/
+    4171,
+/**/
+    4170,
+/**/
+    4169,
+/**/
+    4168,
+/**/
+    4167,
+/**/
+    4166,
+/**/
+    4165,
+/**/
+    4164,
+/**/
+    4163,
+/**/
+    4162,
+/**/
+    4161,
+/**/
+    4160,
+/**/
+    4159,
+/**/
+    4158,
+/**/
+    4157,
+/**/
+    4156,
+/**/
+    4155,
+/**/
+    4154,
+/**/
+    4153,
+/**/
+    4152,
+/**/
+    4151,
+/**/
+    4150,
+/**/
+    4149,
+/**/
+    4148,
+/**/
+    4147,
+/**/
+    4146,
+/**/
+    4145,
+/**/
+    4144,
+/**/
+    4143,
+/**/
+    4142,
+/**/
+    4141,
+/**/
+    4140,
+/**/
+    4139,
+/**/
+    4138,
+/**/
+    4137,
+/**/
+    4136,
+/**/
+    4135,
+/**/
+    4134,
+/**/
+    4133,
+/**/
+    4132,
+/**/
+    4131,
+/**/
+    4130,
+/**/
+    4129,
+/**/
+    4128,
+/**/
+    4127,
+/**/
+    4126,
+/**/
+    4125,
+/**/
+    4124,
+/**/
+    4123,
+/**/
+    4122,
+/**/
+    4121,
+/**/
+    4120,
+/**/
+    4119,
+/**/
+    4118,
+/**/
+    4117,
+/**/
+    4116,
+/**/
+    4115,
+/**/
+    4114,
+/**/
+    4113,
+/**/
+    4112,
+/**/
+    4111,
+/**/
+    4110,
+/**/
+    4109,
+/**/
+    4108,
+/**/
+    4107,
+/**/
+    4106,
+/**/
+    4105,
+/**/
+    4104,
+/**/
+    4103,
+/**/
+    4102,
+/**/
+    4101,
+/**/
+    4100,
+/**/
+    4099,
+/**/
+    4098,
+/**/
+    4097,
+/**/
+    4096,
+/**/
+    4095,
+/**/
+    4094,
+/**/
+    4093,
+/**/
+    4092,
+/**/
+    4091,
+/**/
+    4090,
+/**/
+    4089,
+/**/
+    4088,
+/**/
+    4087,
+/**/
+    4086,
+/**/
+    4085,
+/**/
+    4084,
+/**/
+    4083,
+/**/
+    4082,
+/**/
+    4081,
+/**/
+    4080,
+/**/
+    4079,
+/**/
+    4078,
+/**/
+    4077,
+/**/
+    4076,
+/**/
+    4075,
+/**/
+    4074,
+/**/
+    4073,
+/**/
+    4072,
+/**/
+    4071,
+/**/
+    4070,
+/**/
+    4069,
+/**/
+    4068,
+/**/
+    4067,
+/**/
+    4066,
+/**/
+    4065,
+/**/
+    4064,
+/**/
+    4063,
+/**/
+    4062,
+/**/
+    4061,
+/**/
+    4060,
+/**/
+    4059,
+/**/
+    4058,
+/**/
+    4057,
+/**/
+    4056,
+/**/
+    4055,
+/**/
+    4054,
+/**/
+    4053,
+/**/
+    4052,
+/**/
+    4051,
+/**/
+    4050,
+/**/
+    4049,
+/**/
+    4048,
+/**/
+    4047,
+/**/
+    4046,
+/**/
+    4045,
+/**/
+    4044,
+/**/
+    4043,
+/**/
+    4042,
+/**/
+    4041,
+/**/
+    4040,
+/**/
+    4039,
+/**/
+    4038,
+/**/
+    4037,
+/**/
+    4036,
+/**/
+    4035,
+/**/
+    4034,
+/**/
+    4033,
+/**/
+    4032,
+/**/
+    4031,
+/**/
+    4030,
+/**/
+    4029,
+/**/
+    4028,
+/**/
+    4027,
+/**/
+    4026,
+/**/
+    4025,
+/**/
+    4024,
+/**/
+    4023,
+/**/
+    4022,
+/**/
+    4021,
+/**/
+    4020,
+/**/
+    4019,
+/**/
+    4018,
+/**/
+    4017,
+/**/
+    4016,
+/**/
+    4015,
+/**/
+    4014,
+/**/
+    4013,
+/**/
+    4012,
+/**/
+    4011,
+/**/
+    4010,
+/**/
+    4009,
+/**/
+    4008,
+/**/
+    4007,
+/**/
+    4006,
+/**/
+    4005,
+/**/
+    4004,
+/**/
+    4003,
+/**/
+    4002,
+/**/
+    4001,
+/**/
+    4000,
+/**/
+    3999,
+/**/
+    3998,
+/**/
+    3997,
+/**/
+    3996,
 /**/
     3995,
 /**/
@@ -8886,7 +9878,7 @@ list_in_columns(char_u **items, int size, int current)
     // The rightmost column doesn't need a separator.
     // Sacrifice it to fit in one more column if possible.
     ncol = (int) (Columns + 1) / width;
-    nrow = item_count / ncol + (item_count % ncol ? 1 : 0);
+    nrow = item_count / ncol + ((item_count % ncol) ? 1 : 0);
 
     // "i" counts columns then rows.  "idx" counts rows then columns.
     for (i = 0; !got_int && i < nrow * ncol; ++i)
